using System.Collections.Generic;
using BeginningtoMock.Part2.Business;
using MbUnit.Framework;
using Rhino.Mocks;

namespace BeginningtoMock.Part2.Tests
{
    /// <summary>
    /// Tests the book service code.
    /// </summary>
    [TestFixture]
    public class BookServiceTests
    {
        private MockRepository mocks;
        private List<Book> tempMockDatabase;
        private IBooksDataAccess mockDataAccess;
        private BookService bookService;

        [SetUp]
        public void TestSetup()
        {
            mocks = new MockRepository();
            mockDataAccess = mocks.CreateMock<IBooksDataAccess>();
            tempMockDatabase = new List<Book>();
            bookService = new BookService(mockDataAccess);
        }

        [TearDown]
        public void TestTeardown()
        {
            mocks = null;
            tempMockDatabase = null;
            bookService = null;
            mockDataAccess = null;
        }

        [Test]
        public void GetListOfBooks()
        {
            Book expectedBook = new Book(1, "BeginningtoMock Part 2", 1.00);
            tempMockDatabase.Add(expectedBook);

            using (mocks.Record())
            {
                Expect.Call(mockDataAccess.GetAllBooks()).Return(tempMockDatabase);
            }

            using (mocks.Playback())
            {
                List<Book> bookList = bookService.GetAllBooks();

                Assert.AreEqual(1, bookList.Count);

                Book actualBook = bookList[0];

                Assert.AreEqual(expectedBook, actualBook);
            }
        }

        [Test]
        public void GetBooksBelow10()
        {
            Book expectedBook = new Book(1, "BeginningtoMock Part 2", 1.00);
            tempMockDatabase.Add(expectedBook);

            using (mocks.Record())
            {
                Expect.Call(mockDataAccess.GetAllBooks()).Return(tempMockDatabase);
            }

            using (mocks.Playback())
            {
                List<Book> bookList = bookService.GetBooksWithPriceBelow(10);

                Assert.AreEqual(1, bookList.Count);

                Book actualBook = bookList[0];

                Assert.AreEqual(expectedBook, actualBook);
            }
        }

        [Test]
        public void GetBooksBelow10DoesntReturnAnyAbove()
        {
            Book notExpected = new Book(2, "BeginningtoMock Part 2", 11.00);
            tempMockDatabase.Add(notExpected);

            using (mocks.Record())
            {
                Expect.Call(mockDataAccess.GetAllBooks()).Return(tempMockDatabase);
            }

            using (mocks.Playback())
            {
                List<Book> bookList = bookService.GetBooksWithPriceBelow(10);

                Assert.AreEqual(0, bookList.Count);
            }
        }

        [Test]
        public void GetBooksWherePriceIsExtractlySameAsParameter()
        {
            double bookPrice = 10.15;

            Book expectedBook = new Book(3, "BeginningtoMock Part 2", bookPrice);
            tempMockDatabase.Add(expectedBook);

            using (mocks.Record())
            {
                Expect.Call(mockDataAccess.GetAllBooks()).Return(tempMockDatabase);
            }

            using (mocks.Playback())
            {
                List<Book> bookList = bookService.GetBooksWithPriceBelow(bookPrice);

                Assert.AreEqual(1, bookList.Count);
            }
        }

        [Test]
        public void GetBooksBelow10DoesntReturnAnyAboveButReturnsBelow()
        {
            Book notExpected = new Book(1, "BeginningtoMock Part 2", 11.00);
            tempMockDatabase.Add(notExpected);

            Book expected = new Book(2, "BeginningtoMock Part 2", 10.10);
            tempMockDatabase.Add(expected);


            using (mocks.Record())
            {
                Expect.Call(mockDataAccess.GetAllBooks()).Return(tempMockDatabase);
            }

            using (mocks.Playback())
            {
                List<Book> bookList = bookService.GetBooksWithPriceBelow(10.50);

                Assert.AreEqual(1, bookList.Count);
            }
        }

        [Test]
        [ExpectedException(typeof(CannotConnectToDatabaseException))]
        public void BookServiceHandlesDatabaseExceptionsCorrectly()
        {
            CannotConnectToDatabaseException expectedException = new CannotConnectToDatabaseException("Unable to connect to the database.");
            using (mocks.Record())
            {
                Expect.Call(mockDataAccess.GetAllBooks()).Throw(expectedException);
            }

            using (mocks.Playback())
            {
                bookService.GetAllBooks();
            }
        }
    }
}
